@@ -28,7 +28,12 @@ from types import FunctionType
 
 from hypothesis import strategies as st
 from hypothesis.errors import InvalidArgument, ResolutionFailed
-from hypothesis.internal.compat import PYPY
+from hypothesis.internal.compat import (
+    PYPY,
+    BaseExceptionGroup,
+    ExceptionGroup,
+    get_origin,
+)
 from hypothesis.internal.conjecture.utils import many as conjecture_utils_many
 from hypothesis.strategies._internal.datetime import zoneinfo  # type: ignore
 from hypothesis.strategies._internal.ipaddress import (
@@ -39,11 +44,13 @@ from hypothesis.strategies._internal.ipaddress import (
 from hypothesis.strategies._internal.lazy import unwrap_strategies
 from hypothesis.strategies._internal.strategies import OneOfStrategy
 
+GenericAlias: typing.Any
 UnionType: typing.Any
 try:
     # The type of PEP-604 unions (`int | str`), added in Python 3.10
-    from types import UnionType
+    from types import GenericAlias, UnionType
 except ImportError:
+    GenericAlias = ()
     UnionType = ()
 
 try:
@@ -51,11 +58,6 @@ try:
 except ImportError:
     typing_extensions = None  # type: ignore
 
-try:
-    from typing import _GenericAlias  # type: ignore  # python >= 3.7
-except ImportError:
-    _GenericAlias = ()
-
 try:
     from typing import _AnnotatedAlias  # type: ignore
 except ImportError:
@@ -67,7 +69,7 @@ except ImportError:
 TypeAliasTypes: tuple = ()
 try:
     TypeAliasTypes += (typing.TypeAlias,)
-except AttributeError:
+except AttributeError:  # pragma: no cover
     pass  # Is missing for `python<3.10`
 try:
     TypeAliasTypes += (typing_extensions.TypeAlias,)
@@ -90,6 +92,59 @@ try:
 except AttributeError:  # pragma: no cover
     pass  # `typing_extensions` might not be installed
 
+ConcatenateTypes: tuple = ()
+try:
+    ConcatenateTypes += (typing.Concatenate,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    ConcatenateTypes += (typing_extensions.Concatenate,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+ParamSpecTypes: tuple = ()
+try:
+    ParamSpecTypes += (typing.ParamSpec,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    ParamSpecTypes += (typing_extensions.ParamSpec,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+TypeGuardTypes: tuple = ()
+try:
+    TypeGuardTypes += (typing.TypeGuard,)
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.8`
+try:
+    TypeGuardTypes += (typing_extensions.TypeGuard,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
+RequiredTypes: tuple = ()
+try:
+    RequiredTypes += (typing.Required,)  # type: ignore
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.11`
+try:
+    RequiredTypes += (typing_extensions.Required,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
+NotRequiredTypes: tuple = ()
+try:
+    NotRequiredTypes += (typing.NotRequired,)  # type: ignore
+except AttributeError:  # pragma: no cover
+    pass  # Is missing for `python<3.11`
+try:
+    NotRequiredTypes += (typing_extensions.NotRequired,)
+except AttributeError:  # pragma: no cover
+    pass  # `typing_extensions` might not be installed
+
+
 # We use this variable to be sure that we are working with a type from `typing`:
 typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 
@@ -98,13 +153,34 @@ typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
 # and are just added for more fancy type annotations.
 # `Final` is a great example: it just indicates
 # that this value can't be reassigned.
-NON_RUNTIME_TYPES = frozenset(
-    (
-        *ClassVarTypes,
-        *TypeAliasTypes,
-        *FinalTypes,
-    )
+NON_RUNTIME_TYPES = (
+    typing.Any,
+    *ClassVarTypes,
+    *TypeAliasTypes,
+    *FinalTypes,
+    *ConcatenateTypes,
+    *ParamSpecTypes,
+    *TypeGuardTypes,
 )
+for name in (
+    "Annotated",
+    "NoReturn",
+    "Self",
+    "Required",
+    "NotRequired",
+    "Never",
+    "TypeVarTuple",
+    "Unpack",
+    "LiteralString",
+):
+    try:
+        NON_RUNTIME_TYPES += (getattr(typing, name),)
+    except AttributeError:
+        pass
+    try:
+        NON_RUNTIME_TYPES += (getattr(typing_extensions, name),)
+    except AttributeError:  # pragma: no cover
+        pass  # typing_extensions might not be installed
 
 
 def type_sorting_key(t):
@@ -152,7 +228,7 @@ def try_issubclass(thing, superclass):
             # and Hypothesis issue #2951 closely first, and good luck.  The tests
             # will help you, I hope - good luck.
             if getattr(thing, "__args__", None) is not None:
-                return True
+                return True  # pragma: no cover  # only possible on Python <= 3.9
             for orig_base in getattr(thing, "__orig_bases__", None) or [None]:
                 args = getattr(orig_base, "__args__", None)
                 if _compatible_args(args, superclass_args):
@@ -167,7 +243,7 @@ def is_a_new_type(thing):
     if not isinstance(typing.NewType, type):
         # At runtime, `typing.NewType` returns an identity function rather
         # than an actual type, but we can check whether that thing matches.
-        return (
+        return (  # pragma: no cover  # Python <= 3.9 only
             hasattr(thing, "__supertype__")
             and getattr(thing, "__module__", None) in ("typing", "typing_extensions")
             and inspect.isfunction(thing)
@@ -224,7 +300,7 @@ def find_annotated_strategy(annotated_type):  # pragma: no cover
 def has_type_arguments(type_):
     """Decides whethere or not this type has applied type arguments."""
     args = getattr(type_, "__args__", None)
-    if args and isinstance(type_, _GenericAlias):
+    if args and isinstance(type_, (typing._GenericAlias, GenericAlias)):
         # There are some cases when declared types do already have type arguments
         # Like `Sequence`, that is `_GenericAlias(abc.Sequence[T])[T]`
         parameters = getattr(type_, "__parameters__", None)
@@ -238,7 +314,7 @@ def is_generic_type(type_):
     # The ugly truth is that `MyClass`, `MyClass[T]`, and `MyClass[int]` are very different.
     # We check for `MyClass[T]` and `MyClass[int]` with the first condition,
     # while the second condition is for `MyClass`.
-    return isinstance(type_, typing_root_type) or (
+    return isinstance(type_, typing_root_type + (GenericAlias,)) or (
         isinstance(type_, type) and typing.Generic in type_.__mro__
     )
 
@@ -290,7 +366,8 @@ def from_typing_type(thing):
         literals = []
         while args_dfs_stack:
             arg = args_dfs_stack.pop()
-            if is_typing_literal(arg):
+            if is_typing_literal(arg):  # pragma: no cover
+                # Python 3.10+ flattens for us when constructing Literal objects
                 args_dfs_stack.extend(reversed(arg.__args__))
             else:
                 literals.append(arg)
@@ -351,7 +428,7 @@ def from_typing_type(thing):
         else:
             union_elems = ()
         if not any(
-            isinstance(T, type) and issubclass(int, T)
+            isinstance(T, type) and issubclass(int, get_origin(T) or T)
             for T in list(union_elems) + [elem_type]
         ):
             mapping.pop(typing.ByteString, None)
@@ -480,10 +557,21 @@ _global_type_lookup: typing.Dict[
     UnicodeTranslateError: st.builds(
         UnicodeTranslateError, st.text(), st.just(0), st.just(0), st.just("reason")
     ),
+    BaseExceptionGroup: st.builds(
+        BaseExceptionGroup,
+        st.text(),
+        st.lists(st.from_type(BaseException), min_size=1),
+    ),
+    ExceptionGroup: st.builds(
+        ExceptionGroup,
+        st.text(),
+        st.lists(st.from_type(Exception), min_size=1),
+    ),
     enumerate: st.builds(enumerate, st.just(())),
     filter: st.builds(filter, st.just(lambda _: None), st.just(())),
     map: st.builds(map, st.just(lambda _: None), st.just(())),
     reversed: st.builds(reversed, st.just(())),
+    property: st.builds(property, st.just(lambda _: None)),
     classmethod: st.builds(classmethod, st.just(lambda self: self)),
     staticmethod: st.builds(staticmethod, st.just(lambda self: self)),
     super: st.builds(super, st.from_type(type)),
@@ -495,43 +583,17 @@ if zoneinfo is not None:  # pragma: no branch
     _global_type_lookup[zoneinfo.ZoneInfo] = st.timezones()
 if PYPY:
     _global_type_lookup[builtins.sequenceiterator] = st.builds(iter, st.tuples())  # type: ignore
-try:
-    BaseExceptionGroup  # type: ignore # noqa
-except NameError:
-    pass
-else:  # pragma: no cover
-    _global_type_lookup[BaseExceptionGroup] = st.builds(  # type: ignore
-        BaseExceptionGroup,  # type: ignore
-        st.text(),
-        st.lists(st.from_type(BaseException), min_size=1),
-    )
-    _global_type_lookup[ExceptionGroup] = st.builds(  # type: ignore
-        ExceptionGroup,  # type: ignore
-        st.text(),
-        st.lists(st.from_type(Exception), min_size=1),
-    )
 
 
 _global_type_lookup[type] = st.sampled_from(
     [type(None)] + sorted(_global_type_lookup, key=str)
 )
-if sys.version_info[:2] >= (3, 9):  # pragma: no cover
+if sys.version_info[:2] >= (3, 9):
     # subclass of MutableMapping, and in Python 3.9 we resolve to a union
     # which includes this... but we don't actually ever want to build one.
     _global_type_lookup[os._Environ] = st.just(os.environ)
 
 
-try:  # pragma: no cover
-    import numpy as np
-
-    from hypothesis.extra.numpy import array_dtypes, array_shapes, arrays, scalar_dtypes
-
-    _global_type_lookup[np.dtype] = array_dtypes()
-    _global_type_lookup[np.ndarray] = arrays(scalar_dtypes(), array_shapes(max_dims=2))
-except ImportError:
-    pass
-
-
 _global_type_lookup.update(
     {
         # Note: while ByteString notionally also represents the bytearray and
@@ -594,6 +656,26 @@ if hasattr(typing, "SupportsIndex"):  # pragma: no branch  # new in Python 3.8
     _global_type_lookup[typing.SupportsIndex] = st.integers() | st.booleans()
 
 
+# The "extra" lookups define a callable that either resolves to a strategy for
+# this narrowly extra-specific type, or returns None to proceed with normal
+# type resolution. The callable will only be called if the module is
+# installed. To avoid the performance hit of importing anything here, we defer
+# it until the method is called the first time, at which point we replace the
+# entry in the lookup table with the direct call.
+def _from_numpy_type(thing: typing.Type) -> typing.Optional[st.SearchStrategy]:
+    from hypothesis.extra.numpy import _from_type
+
+    _global_extra_lookup["numpy"] = _from_type
+    return _from_type(thing)
+
+
+_global_extra_lookup: typing.Dict[
+    str, typing.Callable[[typing.Type], typing.Optional[st.SearchStrategy]]
+] = {
+    "numpy": _from_numpy_type,
+}
+
+
 def register(type_, fallback=None, *, module=typing):
     if isinstance(type_, str):
         # Use the name of generic types which are not available on all
@@ -747,7 +829,9 @@ def resolve_OrderedDict(thing):
 
 @register(typing.Pattern, st.builds(re.compile, st.sampled_from(["", b""])))
 def resolve_Pattern(thing):
-    if isinstance(thing.__args__[0], typing.TypeVar):
+    if isinstance(thing.__args__[0], typing.TypeVar):  # pragma: no cover
+        # TODO: this was covered on Python 3.8, but isn't on 3.10 - we should
+        # work out why not and write some extra tests to help avoid regressions.
         return st.builds(re.compile, st.sampled_from(["", b""]))
     return st.just(re.compile(thing.__args__[0]()))
 
@@ -796,13 +880,33 @@ def resolve_Callable(thing):
     # use of keyword arguments and we'd rather not force positional-only.
     if not thing.__args__:  # pragma: no cover  # varies by minor version
         return st.functions()
+
+    *args_types, return_type = thing.__args__
+
     # Note that a list can only appear in __args__ under Python 3.9 with the
     # collections.abc version; see https://bugs.python.org/issue42195
+    if len(args_types) == 1 and isinstance(args_types[0], list):
+        args_types = tuple(args_types[0])  # pragma: no cover
+
+    pep612 = ConcatenateTypes + ParamSpecTypes
+    for arg in args_types:
+        # awkward dance because you can't use Concatenate in isistance or issubclass
+        if getattr(arg, "__origin__", arg) in pep612 or type(arg) in pep612:
+            raise InvalidArgument(
+                "Hypothesis can't yet construct a strategy for instances of a "
+                f"Callable type parametrized by {arg!r}.  Consider using an "
+                "explicit strategy, or opening an issue."
+            )
+    if getattr(return_type, "__origin__", None) in TypeGuardTypes:
+        raise InvalidArgument(
+            "Hypothesis cannot yet construct a strategy for callables which "
+            f"are PEP-647 TypeGuards (got {return_type!r}).  "
+            "Consider using an explicit strategy, or opening an issue."
+        )
+
     return st.functions(
-        like=(lambda: None)
-        if len(thing.__args__) == 1 or thing.__args__[0] == []
-        else (lambda *a, **k: None),
-        returns=st.from_type(thing.__args__[-1]),
+        like=(lambda *a, **k: None) if args_types else (lambda: None),
+        returns=st.from_type(return_type),
     )
 
 
